@@ -5,6 +5,21 @@
 #include "Ema.h"
 #include <stdio.h>
 #include <cmath>
+#include <pymem.h>
+//#include <numpy/ndarraytypes.h>
+//#include "numpy/arrayobject.h"
+//#include <numpy/npy_common.h>
+//#include "npy_config.h"
+#if defined(_WIN32) && !defined(__GNUC__)
+#include <../Lib/site-packages/numpy/core/include/numpy/ndarraytypes.h>
+#include <../Lib/site-packages/numpy/core/include/numpy/arrayobject.h>
+#include <../Lib/site-packages/numpy/core/include/numpy/npy_common.h>
+#else
+#include <numpy/ndarraytypes.h>
+#include <numpy/arrayobject.h>
+#include <numpy/npy_common.h>
+#endif
+//#include <../Lib/site-packages/numpy/npy_config.h>
 
 #define LOGGING(...)
 //#define LOGGING printf
@@ -973,6 +988,12 @@ public:
       // Default every LastTime bin to 0, including floats
       memset(pLastTime, 0, size);
 
+      size = (numUnique + GB_BASE_INDEX) * sizeof(T);
+      T* pLastValue = (T*)WORKSPACE_ALLOC(size);
+
+      // Default every LastValue bin to 0, including floats
+      memset(pLastValue, 0, size);
+
       U Invalid = GET_INVALID(pDest[0]);
 
       // Neiman's matlab loop below
@@ -1025,9 +1046,15 @@ public:
                   // NOTE: fill in last value
                   if (pIncludeMask[i] != 0) {
                      value = pSrc[i];
-                     pLastEma[location] = value + pLastEma[location] * exp(-decayRate * (pTime[i] - pLastTime[location]));
-                     pLastTime[location] = pTime[i];
                   }
+                  else {
+                     // Acts like fill forward
+                     LOGGING("fill forward location: %lld\n", (long long)location);
+                     value = pLastValue[location];
+                  }
+                  pLastEma[location] = value + pLastEma[location] * exp(-decayRate * (pTime[i] - pLastTime[location]));
+                  pLastTime[location] = pTime[i];
+                  pLastValue[location] = value;
                   pDest[i] = pLastEma[location];
                }
                else {
@@ -1079,6 +1106,7 @@ public:
 
       WORKSPACE_FREE(pLastTime);
       WORKSPACE_FREE(pLastEma);
+      WORKSPACE_FREE(pLastValue);
    }
 
 
@@ -1142,6 +1170,12 @@ public:
       size = (numUnique + GB_BASE_INDEX) * sizeof(V);
       V* pLastTime = (V*)WORKSPACE_ALLOC(size);
 
+      size = (numUnique + GB_BASE_INDEX) * sizeof(T);
+      T* pLastValue = (T*)WORKSPACE_ALLOC(size);
+
+      // Default every LastValue bin to 0, including floats
+      memset(pLastValue, 0, size);
+
       // Default every LastTime bin to 0, including floats
       // Set first time to very low value
       V largeNegative = 0;
@@ -1202,12 +1236,14 @@ public:
                   // NOTE: fill in last value
                   if (pIncludeMask[i] != 0) {
                      value = pSrc[i];
-                     EMA_NORMAL_FUNC
                   }
                   else {
-                     pDest[i] = pLastEma[location];
-
+                     // Acts like fill forward
+                     LOGGING("fill forward location: %lld\n", (long long)location);
+                     value = pLastValue[location];
                   }
+                  EMA_NORMAL_FUNC
+                  pLastValue[location] = value;
                }
                else {
                   pDest[i] = Invalid;
@@ -1256,6 +1292,7 @@ public:
 
       WORKSPACE_FREE(pLastTime);
       WORKSPACE_FREE(pLastEma);
+      WORKSPACE_FREE(pLastValue);
    }
 
 
